--- loncom/xml/lontexconvert.pm	2010/01/26 01:16:30	1.101
+++ loncom/xml/lontexconvert.pm	2011/05/27 18:34:07	1.107
@@ -1,7 +1,7 @@
 # The LearningOnline Network with CAPA
 # TeX Conversion Module
 #
-# $Id: lontexconvert.pm,v 1.101 2010/01/26 01:16:30 faziophi Exp $
+# $Id: lontexconvert.pm,v 1.107 2011/05/27 18:34:07 raeburn Exp $
 #
 # Copyright Michigan State University Board of Trustees
 #
@@ -49,7 +49,6 @@ use lib '/home/httpd/lib/perl/';
 use LONCAPA;
 use URI::Escape;
 use IO::Socket::INET;
-use IO::Socket::SSL;
 
 # ====================================================================== Header
 
@@ -166,10 +165,12 @@ sub clean_out_math_mode {
 
 sub displaystyle {
     my ($texstring)=@_;
     #has a $$ or \[ or \displaystyle in it, guessinng it's display mode
     if ($$texstring=~/[^\\]\$\$/ ||
-	$$texstring=~/\\\[/ ||
-	$$texstring=~/\\displaystyle/) { return 1; }
+        $$texstring=~/\\\[/ ||
+        $$texstring=~/\\displaystyle/ ||
+        $$texstring=~/eqnarray/
+       ) { return 1; }
     return 0;
 }
 
@@ -248,19 +249,12 @@ sub mimetex_valign {
     my $valign = 0;
     my $lonhost = $Apache::lonnet::perlvar{'lonHostID'};
     my $hostname = &Apache::lonnet::hostname($lonhost);
-    my $protocol = $Apache::lonnet::protocol{$lonhost};
     my $path = '/cgi-bin/mimetex.cgi?'.$esc_texstring;
     my $socket;
     &Apache::lonxml::start_alarm();
-    if ($protocol eq 'https') {
-        $socket = IO::Socket::SSL->new(PeerAddr => $hostname,
-                                       PeerPort => 'https(443)',
-                                       Proto    => 'tcp');
-    } else {
-        $socket = IO::Socket::INET->new(PeerAddr => $hostname,
-                                        PeerPort => 'http(80)',
-                                        Proto    => 'tcp');
-    }
+    $socket = IO::Socket::INET->new(PeerAddr => $hostname,
+                                    PeerPort => 'http(80)',
+                                    Proto    => 'tcp');
     if ($socket) {
         my $headreq = "HEAD $path HTTP/1.0\r\n\r\n";
         print $socket $headreq;
@@ -269,12 +263,7 @@ sub mimetex_valign {
                 $valign = $1;
             }
         }
-        if ($protocol eq 'https') {
-            $socket->close(SSL_no_shutdown => 1,
-                           SSL_ctx_free => 1);
-        } else {
-            $socket->close();
-        }
+        $socket->close();
     }
     &Apache::lonxml::end_alarm();
     return $valign;
@@ -302,7 +291,7 @@ sub mimetex_converted {
     my $valign = &mimetex_valign($esc_texstring);
     my $result='<img src="/cgi-bin/mimetex.cgi?'.$esc_texstring.'" style="vertical-align:'.$valign.'px" alt="'.$alt_string.'" />';
     if ($displaystyle) {
-	$result='<center>'.$result.'</center>';
+	$result='<div style="text-align:center">'.$result.'</div>';
     }
     return $result;
 }
@@ -326,6 +315,7 @@ sub converted {
 
 sub to_convert {
     my ($string) = @_;
+    &init_tth();
     $string=~s/\<br\s*\/?\>/ /gs;
 #    $string=~s/\s/ /gs;
     $string=&HTML::Entities::decode($string);
@@ -334,7 +324,6 @@ sub to_convert {
 
 sub smiley {
     my $expression=shift;
-    if ($env{'browser.imagesuppress'} eq 'on') { return $expression; }
     my %smileys=(
     	 '\:\-*\)' => 'face-smile.png',
 		 '8\-\)'  => 'face-cool.png',
@@ -373,7 +362,6 @@ sub smiley {
 sub msgtexconverted {
     my ($message,$email) = @_;
     $errorstring='';
-    &init_tth();
     my $outmessage='';
     my $tex=0;
     foreach my $fragment (split(/(?:\&lt\;|\<)\/*m\s*(?:\&gt\;|\>)/i,$message)) {
@@ -394,15 +382,16 @@ sub msgtexconverted {
     foreach my $fragment (split(/(?:\&lt\;|\<)\/*algebra\s*(?:\&gt\;|\>)/i,
 				$message)) {
 	if ($tex) {
+        my $algebra = &algebra($fragment, 'web', undef, undef, undef, undef, 'tth');
 	    if ($email) {
-		$outmessage.='</pre><tt>'.&algebra($fragment,'web').'</tt><pre>';
+		$outmessage.='</pre><tt>'.$algebra.'</tt><pre>';
 		$tex=0;
 	    } else {
-		$outmessage.=&algebra($fragment,'web');
+		$outmessage.=$algebra;
 		$tex=0;
 	    }
 	} else {
-            $outmessage.=$fragment;
+        $outmessage.=$fragment;
 	    $tex=1;
 	}
     }
@@ -415,9 +404,9 @@ sub msgtexconverted {
 
 sub algebra {
     use AlgParser;
-
-    my ($string,$target,$style,$parstack,$safeeval)=@_;
+    my ($string,$target,$style,$parstack,$safeeval,$tth)=@_;
     my $parser = new AlgParserWithImplicitExpand;
+    if ($tth eq 'tth') {&init_tth();}
     $string=&prepare_algebra($string);
     my $ret = $parser->parse($string);
     my $result='['.&mt('Algebra unconverted due to previous errors').']';
@@ -528,6 +517,39 @@ Header
 
 =item init_math_support()
 
+=item mimetex_valign()
+
+ Makes a HEAD call to /cgi-bin/mimetex.cgi via IO:: to retrieve the 
+ vertical alignment, before the subsequent call to mimetex_converted()
+ which generates the <img> tag and the corresponding image.
+
+ Input: 1.  $esc_texstring (escaped TeX to be rendered by mimetex).
+ Output: 1. $valign - number of pixels: positive or negative integer 
+            which will be included in <img> tag for mimetex image to
+            support vertical alignment of image within a line of text.
+
+ If a server is running SSL, and Apache rewrite rules are in place 
+ to rewrite requests for http to https, modification will most likely 
+ be needed for pass through for HEAD requests for /cgi-bin/mimetex.cgi. 
+
+ Example rewrite rules which rewrite all http traffic to https, 
+ except HEAD requests for /cgi-bin/mimetex.cgi are:
+
+ <IfModule mod_rewrite.c>
+     RewriteEngine On
+     RewriteLogLevel 0
+
+     RewriteCond %{HTTPS} off
+     RewriteCond %{HTTP:Host} (.*)
+     RewriteCond %{REQUEST_METHOD} !HEAD 
+     RewriteRule ^/(.*) https://%1/$1 [R=301,L]
+
+     RewriteCond %{HTTPS} off
+     RewriteCond %{HTTP:Host} (.*)
+     RewriteCond %{REQUEST_METHOD} HEAD
+     RewriteCond %{REQUEST_URI} !^/cgi-bin/mimetex.cgi
+     RewriteRule ^/(.*) https://%1/$1 [R=301,L]
+ </IfModule>
 
 =item mimetex_converted()
@@ -166,10 +165,12 @@ sub clean_out_math_mode {
 sub displaystyle {
     my ($texstring)=@_;
-    #has a $$ or \[ or \displaystyle in it, guessinng it's display mode
+    #has a $$ or \[ or \displaystyle or eqnarray in it, guessinng it's display mode
     if ($$texstring=~/[^\\]\$\$/ ||